"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "Box", {
  enumerable: true,
  get: function () {
    return _Box.default;
  }
});
Object.defineProperty(exports, "Text", {
  enumerable: true,
  get: function () {
    return _Text.default;
  }
});
Object.defineProperty(exports, "Color", {
  enumerable: true,
  get: function () {
    return _Color.default;
  }
});
Object.defineProperty(exports, "StdinContext", {
  enumerable: true,
  get: function () {
    return _StdinContext.default;
  }
});
Object.defineProperty(exports, "StdoutContext", {
  enumerable: true,
  get: function () {
    return _StdoutContext.default;
  }
});
exports.render = void 0;

var _react = _interopRequireDefault(require("react"));

var _undom = _interopRequireDefault(require("undom"));

var _logUpdate = _interopRequireDefault(require("log-update"));

var _debounceFn = _interopRequireDefault(require("debounce-fn"));

var _createReconciler = _interopRequireDefault(require("./create-reconciler"));

var _createRenderer = _interopRequireDefault(require("./create-renderer"));

var _App = _interopRequireDefault(require("./components/App"));

var _Box = _interopRequireDefault(require("./components/Box"));

var _Text = _interopRequireDefault(require("./components/Text"));

var _Color = _interopRequireDefault(require("./components/Color"));

var _StdinContext = _interopRequireDefault(require("./components/StdinContext"));

var _StdoutContext = _interopRequireDefault(require("./components/StdoutContext"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const render = (node, options = {}) => {
  if (typeof options.write === 'function') {
    options = {
      stdout: options,
      stdin: process.stdin
    };
  }

  options = _objectSpread({
    stdout: process.stdout,
    stdin: process.stdin,
    debug: false
  }, options);
  const document = (0, _undom.default)();
  const render = (0, _createRenderer.default)({
    terminalWidth: options.stdout.columns
  });

  const log = _logUpdate.default.create(options.stdout); // Ignore last render after unmounting a tree to prevent empty output before exit


  let ignoreRender = false;

  const onRender = () => {
    if (ignoreRender) {
      return;
    }

    const output = render(document.body);

    if (options.debug) {
      options.stdout.write(output);
      return;
    }

    log(output);
  };

  const debouncedRender = options.debug ? onRender : (0, _debounceFn.default)(onRender);
  const reconciler = (0, _createReconciler.default)(document, debouncedRender);

  if (!options.stdout._inkContainer) {
    options.stdout._inkContainer = reconciler.createContainer(document.body, false);
  }

  const tree = _react.default.createElement(_App.default, {
    stdin: options.stdin,
    stdout: options.stdout
  }, node);

  reconciler.updateContainer(tree, options.stdout._inkContainer);
  return () => {
    if (typeof debouncedRender.cancel === 'function') {
      debouncedRender.cancel();
      onRender();
      log.done();
    }

    ignoreRender = true;
    reconciler.updateContainer(null, options.stdout._inkContainer);
  };
};

exports.render = render;